package org.tbasket.handler

import io.getquill.*
import io.getquill.context.ZioJdbc.*
import io.getquill.context.qzio.{ZioContext, ZioJdbcContext}
import io.getquill.context.sql.idiom.SqlIdiom
import org.apache.logging.log4j.LogManager
import org.tbasket.auth.Authentificator
import org.tbasket.data.{DatabaseContext, User}
import org.tbasket.error.AuthException.*
import org.tbasket.error.ExceptionEnum
import org.tbasket.error.JwtException.*
import org.tbasket.error.RegularException.*
import org.tbasket.handler.HandlerUtils.errorBody
import zio.http.*
import zio.http.model.{Cookie, Header, Headers, Status}
import zio.json.*
import zio.json.ast.Json.Str
import zio.json.ast.{Json, JsonCursor}
import zio.{ZEnvironment, ZIO, *}

import java.sql.SQLException
import java.util.UUID


object LoginPageHandler extends PageHandler:
  
  private val LOG = LogManager.getLogger("Login")
  
  private def getUser(json: Json) =
    ZIO.serviceWithZIO[Authentificator] { auth =>
      for
        mail <- HandlerUtils.parseAttribute(json, "email", JsonCursor.field("email").isString)
        password <- HandlerUtils.parseAttribute(json, "password", JsonCursor.field("password").isString)
        user <- auth.loginUser(mail, password)
      yield user
    }
  
  private def tryPost(request: Request) =
    for
      body <- request
        .body
        .asString
        .mapError(s =>
          InvalidRequest("Invalid request body", s.getMessage)
        )
      
      json <- ZIO.fromEither(body.fromJson[Json])
        .mapError(InvalidRequest("Invalid JSON body", _))
      
      user <- getUser(json)
      jwt <- ZIO.serviceWithZIO[Authentificator](_.requestNewJwt(user))
    yield Response(
      status = Status.Found,
      headers = Headers.location("/") ++ //login successful, go back to main page
        Headers.setCookie(Cookie("JWT", jwt)) //and set the token cookie
    )
  
  def post(request: Request) =
    tryPost(request).catchAll {
      case UserNotFound => ZIO.attempt(Response(
        status = Status.Unauthorized,
        body = errorBody("unauthorized", "unknown user email"),
        headers =
          Headers(
            Headers.location("/register")
          ) // send back caller to register panel
      ))
      
      case InvalidPassword => ZIO.attempt(Response(
        status = Status.Unauthorized,
        body = errorBody("unauthorized", "invalid password")
      ))
      
      case InvalidRequest(msg, cause) => ZIO.attempt(Response(
        status = Status.Unauthorized,
        body = errorBody("invalid request", s"$cause: $msg")
      ))
      
      case InternalError(e) =>
        LOG.error("Internal error : ")
        LOG.throwing(e)
        ZIO.attempt(Response(
          status = Status.InternalServerError,
          body = errorBody("internal", "internal error, please contact support")
        ))
      
      case other =>
        LOG.error("Unhandle exception : ")
        LOG.throwing(other)
        ZIO.attempt(Response(
          status = Status.InternalServerError,
          body = errorBody("internal", "internal error, please contact support")
        ))
    }
